
Set
   i        'network buses'    / 1*33   /
   slack(i)                    / 1     /
   GB(i)    'generating buses' / 1,2,3,6,10,15 /;

Scalar
   Sbase /   1000 /
   Vbase /12.66/      ;
parameter Zbase;
Zbase=power(Vbase,2)*1000/Sbase;

Alias (i,j);

Table GenD(i,*) 'generating units characteristics'
         Pmax      Pmin    Qmax    Qmin
*      (kW)      (kW)    (kVAr)  (kVAr)
1      14000       0      1500    -200
2       500        0      200     -100
3       500        0      200     -100
6       500        0      200     -100
10      300        30     200     -100
15      300        30     200     -100


Table BD(i,*) 'demands of each bus in MW'
       Pd   Qd
   1   0    0
   2   100  60
   3   90   40
   4   120  80
   5   60   30
   6   60   20
   7   200  100
   8   200  100
   9   60   20
   10  60   20
   11  45   30
   12  60   35
   13  60   35
   14  120  80
   15  60   10
   16  60   20
   17  60   20
   18  90   40
   19  90   40
   20  90   40
   21  90   40
   22  90   40
   23  90   50
   24  420  200
   25  420  200
   26  60   25
   27  60   25
   28  60   20
   29  120  70
   30  200  600
   31  150  70
   32  210  100
   33  60   40             ;
****************************************************

Table LN(i,j,*) 'network technical characteristics'

          r           x          Limit
*       (ohm)       (ohm)        (KVA)
1.2      0.0192      0.0575       1300
2.3      0.493       0.2511       1300
3.4      0.366       0.1864       650
4.5      0.3811      0.1941       1300
5.6      0.819       0.707        1300
6.7      0.1872      0.6188       650
7.8      0.7114      0.2351       900
8.9      1.03        0.74         700
9.10     1.044       0.74         1300
10.11    0.1966      0.065        320
11.12    0.3744      0.1238       650
12.13    1.468       1.155        320
13.14    0.5416      0.7129       650
14.15    0.591       0.526        650
15.16    07463       0.545        650
16.17    1.289       1.721        650
17.18    0.732       0.574        320
2.19     0.164       0.1565       320
19.20    1.5042      1.3554       320
20.21    0.4095      0.4784       160
21.22    0.7089      0.9373       160
3.23     0.4512      0.3083       160
23.24    0.898       0.7091       160
24.25    0.896       0.7011       320
6.26     0.203       0.1034       320
26.27    0.2842      0.1447       320
27.28    1.059       0.9337       320
28.29    0.8042      0.7006       320
29.30    0.5075      0.2585       320
30.31    0.9744      0.963        160
31.32    0.3105      0.3619       160
32.33    0.341       0.5302       160
;
* ----------------------------------------------



*LN(i,j,'b') = 0;
LN(i,j,'x')$(LN(i,j,'x')=0) = LN(j,i,'x');
LN(i,j,'r')$(LN(i,j,'r')=0) = LN(j,i,'r');
LN(i,j,'b')$(LN(i,j,'b')=0) = LN(j,i,'b');
LN(i,j,'Limit')$(LN(i,j,'Limit')=0) =   LN(j,i,'Limit');
LN(i,j,'z')$LN(i,j,'Limit') = sqrt(sqr(LN(i,j,'x')) + sqr(LN(i,j,'r')));
LN(j,i,'z')$(LN(i,j,'z')=0) = LN(i,j,'z');
LN(i,j,'th')$(LN(i,j,'Limit') and LN(i,j,'x') and LN(i,j,'r'))   = arctan(LN(i,j,'x')/(LN(i,j,'r')));
LN(i,j,'th')$(LN(i,j,'Limit') and LN(i,j,'x') and LN(i,j,'r')=0) = pi/2;
LN(i,j,'th')$(LN(i,j,'Limit') and LN(i,j,'r') and LN(i,j,'x')=0) = 0;
LN(j,i,'th')$LN(i,j,'Limit') = LN(i,j,'th');

Parameter cx(i,j);
cx(i,j)$(LN(i,j,'limit') and LN(j,i,'limit')) = 1;
cx(i,j)$(cx(j,i)) = 1;

Variable OF, Pij(i,j), Qij(i,j), Pg(i), Qg(i), Va(i), V(i),Ploss(i,j),Qloss(i,j);
Equation eq1, eq2, eq3, eq4, eq5,eq6;

eq1(i,j)$cx(i,j)..
   Pij(i,j) =e= (V(i)*V(i)*cos(LN(j,i,'th')) - V(i)*V(j)*cos(Va(i) - Va(j) + LN(j,i,'th')))
                  /(LN(j,i,'z')/Zbase);

eq2(i,j)$cx(i,j)..
   Qij(i,j) =e= (V(i)*V(i)*sin(LN(j,i,'th')) - V(i)*V(j)*sin(Va(i) - Va(j) + LN(j,i,'th')))
                  /(LN(j,i,'z')/Zbase) ;

eq3(i)..
   Pg(i)$GenD(i,'Pmax') -(BD(i,'pd')/Sbase) =G= sum(j$cx(j,i), Pij(i,j));

eq4(i)..
   Qg(i)$GenD(i,'Qmax') - (BD(i,'qd')/Sbase) =G= sum(j$cx(j,i), Qij(i,j));

eq5(i,j)$cx(i,j)..
  Ploss(i,j) =e= (LN(i,j,'r')/LN(i,j,'z'))*(power(V(i),2)+power(V(j),2)-2*V(i)*V(j)*cos(Va(i)-Va(j)));


eq6..
      of=e=sum((i,j)$(cx(i,j)),Ploss(i,j));


Model loadflow / all /;

Pg.lo(i) = GenD(i,'Pmin')/Sbase;
Pg.up(i) = GenD(i,'Pmax')/Sbase;
Qg.lo(i) = GenD(i,'Qmin')/Sbase;
Qg.up(i) = GenD(i,'Qmax')/Sbase;

Va.up(i)     = pi/2;
Va.lo(i)     =-pi/2;
Va.l(i)      = 0;
*Va.fx(slack) = 0;

*Pij.up(i,j)$((cx(i,j))) = 1*LN(i,j,'Limit')/Sbase;
*Pij.lo(i,j)$((cx(i,j))) =-1*LN(i,j,'Limit')/Sbase;
*Qij.up(i,j)$((cx(i,j))) = 1*LN(i,j,'Limit')/Sbase;
*Qij.lo(i,j)$((cx(i,j))) =-1*LN(i,j,'Limit')/Sbase;

V.lo(i)  = 0.95;
V.up(i)  = 1.05;
V.fx(slack) = 1.02;
V.l(i)   = 1;


option nlp=knitro;
option  optcr=.0000001;

solve loadflow minimizing OF using nlp;

Parameter report(i,*), report2(i,j,*),Plos_Total  ;
report(i,'VOLTAGE(pu)')     = V.l(i);
report(i,'Angle(rad)') = Va.l(i);
report(i,'Pg(pu)')    = Pg.l(i);
report(i,'Qg(pu)')    = Qg.l(i);
report2(i,j,'Pij(pu)')=Pij.l(i,j);
report2(i,j,'Qij(pu)')=Qij.l(i,j);
report2(i,j,'Plossij(kw)')=Ploss.l(i,j)*Sbase;
Plos_Total=OF.L*Sbase;

display  report,report2,Plos_Total;

